<?php

namespace Bixo\Account\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Bixo\Account\Http\Requests\BankAccountRequest;
use Bixo\Account\Interfaces\BankAccountRepositoryInterface;
use Bixo\Account\Models\BankAccount;

/**
 * Resource controller class for bank_account.
 */
class BankAccountResourceController extends BaseController
{

    /**
     * Initialize bank_account resource controller.
     *
     * @param type BankAccountRepositoryInterface $bank_account
     *
     * @return null
     */
    public function __construct(BankAccountRepositoryInterface $bank_account)
    {
        parent::__construct();
        $this->repository = $bank_account;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Bixo\Account\Repositories\Criteria\BankAccountResourceCriteria::class);
    }

    /**
     * Display a list of bank_account.
     *
     * @return Response
     */
    public function index(BankAccountRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Bixo\Account\Repositories\Presenter\BankAccountPresenter::class)
                ->$function();
        }

        $bank_accounts = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('account::bank_account.names'))
            ->view('account::bank_account.index', true)
            ->data(compact('bank_accounts', 'view'))
            ->output();
    }

    /**
     * Display bank_account.
     *
     * @param Request $request
     * @param Model   $bank_account
     *
     * @return Response
     */
    public function show(BankAccountRequest $request, BankAccount $bank_account)
    {

        if ($bank_account->exists) {
            $view = 'account::bank_account.show';
        } else {
            $view = 'account::bank_account.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('account::bank_account.name'))
            ->data(compact('bank_account'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new bank_account.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(BankAccountRequest $request)
    {

        $bank_account = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('account::bank_account.name')) 
            ->view('account::bank_account.create', true) 
            ->data(compact('bank_account'))
            ->output();
    }

    /**
     * Create new bank_account.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(BankAccountRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $bank_account                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('account::bank_account.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('account/bank_account/' . $bank_account->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/account/bank_account'))
                ->redirect();
        }

    }

    /**
     * Show bank_account for editing.
     *
     * @param Request $request
     * @param Model   $bank_account
     *
     * @return Response
     */
    public function edit(BankAccountRequest $request, BankAccount $bank_account)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('account::bank_account.name'))
            ->view('account::bank_account.edit', true)
            ->data(compact('bank_account'))
            ->output();
    }

    /**
     * Update the bank_account.
     *
     * @param Request $request
     * @param Model   $bank_account
     *
     * @return Response
     */
    public function update(BankAccountRequest $request, BankAccount $bank_account)
    {
        try {
            $attributes = $request->all();

            $bank_account->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('account::bank_account.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('account/bank_account/' . $bank_account->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('account/bank_account/' . $bank_account->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the bank_account.
     *
     * @param Model   $bank_account
     *
     * @return Response
     */
    public function destroy(BankAccountRequest $request, BankAccount $bank_account)
    {
        try {

            $bank_account->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('account::bank_account.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('account/bank_account/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('account/bank_account/' . $bank_account->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple bank_account.
     *
     * @param Model   $bank_account
     *
     * @return Response
     */
    public function delete(BankAccountRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('account::bank_account.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('account/bank_account'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/account/bank_account'))
                ->redirect();
        }

    }

    /**
     * Restore deleted bank_accounts.
     *
     * @param Model   $bank_account
     *
     * @return Response
     */
    public function restore(BankAccountRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('account::bank_account.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/account/bank_account'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/account/bank_account/'))
                ->redirect();
        }

    }

}
