<?php

namespace Bixo\Account\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Bixo\Account\Http\Requests\CoaRequest;
use Bixo\Account\Interfaces\CoaRepositoryInterface;
use Bixo\Account\Models\Coa;

/**
 * Resource controller class for coa.
 */
class CoaResourceController extends BaseController
{

    /**
     * Initialize coa resource controller.
     *
     * @param type CoaRepositoryInterface $coa
     *
     * @return null
     */
    public function __construct(CoaRepositoryInterface $coa)
    {
        parent::__construct();
        $this->repository = $coa;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Bixo\Account\Repositories\Criteria\CoaResourceCriteria::class);
    }

    /**
     * Display a list of coa.
     *
     * @return Response
     */
    public function index(CoaRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Bixo\Account\Repositories\Presenter\CoaPresenter::class)
                ->$function();
        }

        $coas = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('account::coa.names'))
            ->view('account::coa.index', true)
            ->data(compact('coas', 'view'))
            ->output();
    }

    /**
     * Display coa.
     *
     * @param Request $request
     * @param Model   $coa
     *
     * @return Response
     */
    public function show(CoaRequest $request, Coa $coa)
    {

        if ($coa->exists) {
            $view = 'account::coa.show';
        } else {
            $view = 'account::coa.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('account::coa.name'))
            ->data(compact('coa'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new coa.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(CoaRequest $request)
    {

        $coa = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('account::coa.name')) 
            ->view('account::coa.create', true) 
            ->data(compact('coa'))
            ->output();
    }

    /**
     * Create new coa.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(CoaRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $coa                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('account::coa.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('account/coa/' . $coa->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/account/coa'))
                ->redirect();
        }

    }

    /**
     * Show coa for editing.
     *
     * @param Request $request
     * @param Model   $coa
     *
     * @return Response
     */
    public function edit(CoaRequest $request, Coa $coa)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('account::coa.name'))
            ->view('account::coa.edit', true)
            ->data(compact('coa'))
            ->output();
    }

    /**
     * Update the coa.
     *
     * @param Request $request
     * @param Model   $coa
     *
     * @return Response
     */
    public function update(CoaRequest $request, Coa $coa)
    {
        try {
            $attributes = $request->all();

            $coa->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('account::coa.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('account/coa/' . $coa->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('account/coa/' . $coa->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the coa.
     *
     * @param Model   $coa
     *
     * @return Response
     */
    public function destroy(CoaRequest $request, Coa $coa)
    {
        try {

            $coa->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('account::coa.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('account/coa/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('account/coa/' . $coa->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple coa.
     *
     * @param Model   $coa
     *
     * @return Response
     */
    public function delete(CoaRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('account::coa.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('account/coa'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/account/coa'))
                ->redirect();
        }

    }

    /**
     * Restore deleted coas.
     *
     * @param Model   $coa
     *
     * @return Response
     */
    public function restore(CoaRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('account::coa.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/account/coa'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/account/coa/'))
                ->redirect();
        }

    }

}
