<?php

namespace Bixo\Account\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Bixo\Account\Http\Requests\InvoiceDetailRequest;
use Bixo\Account\Interfaces\InvoiceDetailRepositoryInterface;
use Bixo\Account\Models\InvoiceDetail;

/**
 * Resource controller class for invoice_detail.
 */
class InvoiceDetailResourceController extends BaseController
{

    /**
     * Initialize invoice_detail resource controller.
     *
     * @param type InvoiceDetailRepositoryInterface $invoice_detail
     *
     * @return null
     */
    public function __construct(InvoiceDetailRepositoryInterface $invoice_detail)
    {
        parent::__construct();
        $this->repository = $invoice_detail;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Bixo\Account\Repositories\Criteria\InvoiceDetailResourceCriteria::class);
    }

    /**
     * Display a list of invoice_detail.
     *
     * @return Response
     */
    public function index(InvoiceDetailRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Bixo\Account\Repositories\Presenter\InvoiceDetailPresenter::class)
                ->$function();
        }

        $invoice_details = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('account::invoice_detail.names'))
            ->view('account::invoice_detail.index', true)
            ->data(compact('invoice_details', 'view'))
            ->output();
    }

    /**
     * Display invoice_detail.
     *
     * @param Request $request
     * @param Model   $invoice_detail
     *
     * @return Response
     */
    public function show(InvoiceDetailRequest $request, InvoiceDetail $invoice_detail)
    {

        if ($invoice_detail->exists) {
            $view = 'account::invoice_detail.show';
        } else {
            $view = 'account::invoice_detail.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('account::invoice_detail.name'))
            ->data(compact('invoice_detail'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new invoice_detail.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(InvoiceDetailRequest $request)
    {

        $invoice_detail = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('account::invoice_detail.name')) 
            ->view('account::invoice_detail.create', true) 
            ->data(compact('invoice_detail'))
            ->output();
    }

    /**
     * Create new invoice_detail.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(InvoiceDetailRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $invoice_detail                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('account::invoice_detail.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('account/invoice_detail/' . $invoice_detail->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/account/invoice_detail'))
                ->redirect();
        }

    }

    /**
     * Show invoice_detail for editing.
     *
     * @param Request $request
     * @param Model   $invoice_detail
     *
     * @return Response
     */
    public function edit(InvoiceDetailRequest $request, InvoiceDetail $invoice_detail)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('account::invoice_detail.name'))
            ->view('account::invoice_detail.edit', true)
            ->data(compact('invoice_detail'))
            ->output();
    }

    /**
     * Update the invoice_detail.
     *
     * @param Request $request
     * @param Model   $invoice_detail
     *
     * @return Response
     */
    public function update(InvoiceDetailRequest $request, InvoiceDetail $invoice_detail)
    {
        try {
            $attributes = $request->all();

            $invoice_detail->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('account::invoice_detail.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('account/invoice_detail/' . $invoice_detail->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('account/invoice_detail/' . $invoice_detail->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the invoice_detail.
     *
     * @param Model   $invoice_detail
     *
     * @return Response
     */
    public function destroy(InvoiceDetailRequest $request, InvoiceDetail $invoice_detail)
    {
        try {

            $invoice_detail->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('account::invoice_detail.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('account/invoice_detail/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('account/invoice_detail/' . $invoice_detail->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple invoice_detail.
     *
     * @param Model   $invoice_detail
     *
     * @return Response
     */
    public function delete(InvoiceDetailRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('account::invoice_detail.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('account/invoice_detail'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/account/invoice_detail'))
                ->redirect();
        }

    }

    /**
     * Restore deleted invoice_details.
     *
     * @param Model   $invoice_detail
     *
     * @return Response
     */
    public function restore(InvoiceDetailRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('account::invoice_detail.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/account/invoice_detail'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/account/invoice_detail/'))
                ->redirect();
        }

    }

}
