<?php

namespace Bixo\Account\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Bixo\Account\Http\Requests\ReceiptRequest;
use Bixo\Account\Interfaces\ReceiptRepositoryInterface;
use Bixo\Account\Models\Receipt;

/**
 * Resource controller class for receipt.
 */
class ReceiptResourceController extends BaseController
{

    /**
     * Initialize receipt resource controller.
     *
     * @param type ReceiptRepositoryInterface $receipt
     *
     * @return null
     */
    public function __construct(ReceiptRepositoryInterface $receipt)
    {
        parent::__construct();
        $this->repository = $receipt;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Bixo\Account\Repositories\Criteria\ReceiptResourceCriteria::class);
    }

    /**
     * Display a list of receipt.
     *
     * @return Response
     */
    public function index(ReceiptRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Bixo\Account\Repositories\Presenter\ReceiptPresenter::class)
                ->$function();
        }

        $receipts = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('account::receipt.names'))
            ->view('account::receipt.index', true)
            ->data(compact('receipts', 'view'))
            ->output();
    }

    /**
     * Display receipt.
     *
     * @param Request $request
     * @param Model   $receipt
     *
     * @return Response
     */
    public function show(ReceiptRequest $request, Receipt $receipt)
    {

        if ($receipt->exists) {
            $view = 'account::receipt.show';
        } else {
            $view = 'account::receipt.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('account::receipt.name'))
            ->data(compact('receipt'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new receipt.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ReceiptRequest $request)
    {

        $receipt = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('account::receipt.name')) 
            ->view('account::receipt.create', true) 
            ->data(compact('receipt'))
            ->output();
    }

    /**
     * Create new receipt.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ReceiptRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $receipt                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('account::receipt.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('account/receipt/' . $receipt->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/account/receipt'))
                ->redirect();
        }

    }

    /**
     * Show receipt for editing.
     *
     * @param Request $request
     * @param Model   $receipt
     *
     * @return Response
     */
    public function edit(ReceiptRequest $request, Receipt $receipt)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('account::receipt.name'))
            ->view('account::receipt.edit', true)
            ->data(compact('receipt'))
            ->output();
    }

    /**
     * Update the receipt.
     *
     * @param Request $request
     * @param Model   $receipt
     *
     * @return Response
     */
    public function update(ReceiptRequest $request, Receipt $receipt)
    {
        try {
            $attributes = $request->all();

            $receipt->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('account::receipt.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('account/receipt/' . $receipt->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('account/receipt/' . $receipt->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the receipt.
     *
     * @param Model   $receipt
     *
     * @return Response
     */
    public function destroy(ReceiptRequest $request, Receipt $receipt)
    {
        try {

            $receipt->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('account::receipt.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('account/receipt/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('account/receipt/' . $receipt->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple receipt.
     *
     * @param Model   $receipt
     *
     * @return Response
     */
    public function delete(ReceiptRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('account::receipt.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('account/receipt'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/account/receipt'))
                ->redirect();
        }

    }

    /**
     * Restore deleted receipts.
     *
     * @param Model   $receipt
     *
     * @return Response
     */
    public function restore(ReceiptRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('account::receipt.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/account/receipt'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/account/receipt/'))
                ->redirect();
        }

    }

}
